@@ -15,7 +15,6 @@ import os
 import sys
 
 SOLUTION_TEMPLATE = """\
-
 Microsoft Visual Studio Solution File, Format Version 12.00
 # Visual Studio Version 16
 VisualStudioVersion = 16.0.28803.352
@@ -42,8 +41,7 @@ SOLUTION_PROJECT_TEMPLATE = """\
 Project("{{8BC9CEB8-8B4A-11D0-8D11-00A0C91BC942}}") = "{NAME}", "{NAME}\\{NAME}.vcxproj", "{{{GUID}}}"
 EndProject"""
 
-PROJECT_TEMPLATE = """\
-<?xml version="1.0" encoding="utf-8"?>
+PROJECT_TEMPLATE = r"""<?xml version="1.0" encoding="utf-8"?>
 <Project DefaultTargets="Build" xmlns="http://schemas.microsoft.com/developer/msbuild/2003">
   <ItemGroup Label="ProjectConfigurations">
     <ProjectConfiguration Include="Build|Win32">
