@@ -4,17 +4,37 @@
 # SPDX-License-Identifier: GPL-3.0-or-later
 
 from .errors import ParseError
-from .tty import WarnOnce
+from .tty import colorize, WarnOnce, WARNING
 import copy
 import errno
 import os
 import pickle
 import sqlite3
+import struct
+import sys
+import zlib
 
 warnNoAttic = WarnOnce(
     "Project was created by old Bob version. Attic directories listing will be incomplete.",
     help="Attic directires were not tracked by Bob version 0.14 and below.")
 
+class DigestAdder:
+    """Append a checksum by compuing it on-the-fly"""
+    def __init__(self, fd):
+        self.fd = fd
+        self.csum = 1
+
+    def write(self, data):
+        self.csum = zlib.adler32(data, self.csum)
+        return self.fd.write(data)
+
+    def __enter__(self):
+        return self
+
+    def __exit__(self, exc_type, exc_value, traceback):
+        self.fd.write(struct.pack("=L", self.csum))
+        return False
+
 class _BobState():
     # Bump CUR_VERSION if internal state is made backwards incompatible, that is
     # older versions ob Bob will choke on the persisted state. The MIN_VERSION
@@ -35,6 +55,7 @@ class _BobState():
     instance = None
     def __init__(self):
         self.__path = ".bob-state.pickle"
+        self.__uncommittedPath = self.__path + ".new"
         self.__byNameDirs = {}
         self.__results = {}
         self.__inputs = {}
@@ -48,6 +69,7 @@ class _BobState():
         self.__variantIds = {}
         self.__atticDirs = {}
         self.__createdWithVersion = self.CUR_VERSION
+        self.__storagePath = {}
 
         # lock state
         lockFile = ".bob-state.lock"
@@ -65,6 +87,10 @@ class _BobState():
             self.__lock = lockFile
             os.close(fd)
 
+        # Commit old state if valid. It may have been left behind if Bob or the
+        # machine crashed really hard.
+        self.__commit()
+
         # load state if it exists
         try:
             if os.path.exists(self.__path):
@@ -73,7 +99,7 @@ class _BobState():
                         state = pickle.load(f)
                 except OSError as e:
                     raise ParseError("Error loading workspace state: " + str(e))
-                except pickle.PickleError as e:
+                except (pickle.PickleError, ValueError) as e:
                     raise ParseError("Error decoding workspace state: " + str(e))
 
                 if state["version"] < _BobState.MIN_VERSION:
@@ -91,6 +117,7 @@ class _BobState():
                 self.__variantIds = state.get("variantIds", {})
                 self.__atticDirs = state.get("atticDirs", {})
                 self.__createdWithVersion = state.get("createdWithVersion", 0)
+                self.__storagePath = state.get("storagePath", {})
 
                 # version upgrades
                 if state["version"] == 2:
@@ -133,20 +160,50 @@ class _BobState():
                 "variantIds" : self.__variantIds,
                 "atticDirs" : self.__atticDirs,
                 "createdWithVersion" : self.__createdWithVersion,
+                "storagePath" : self.__storagePath,
             }
-            tmpFile = self.__path+".new"
             try:
-                with open(tmpFile, "wb") as f:
-                    pickle.dump(state, f)
-                    f.flush()
-                    os.fsync(f.fileno())
-                os.replace(tmpFile, self.__path)
+                with open(self.__uncommittedPath, "wb") as f:
+                    with DigestAdder(f) as df:
+                        pickle.dump(state, df)
             except OSError as e:
                 raise ParseError("Error saving workspace state: " + str(e))
             self.__dirty = False
         else:
             self.__dirty = True
 
+    def __commit(self, verify=True):
+        if not os.path.exists(self.__uncommittedPath):
+            return
+
+        try:
+            commit = True
+            with open(self.__uncommittedPath, "r+b") as f:
+                if verify:
+                    data = f.read()
+                    csum = struct.pack("=L", zlib.adler32(data[:-4]))
+                    commit = (csum == data[-4:])
+                os.fsync(f.fileno())
+            if commit:
+                os.replace(self.__uncommittedPath, self.__path)
+                return
+            else:
+                print(colorize("Warning: discarding corrupted workspace state!", WARNING),
+                    file=sys.stderr)
+                print("You might experience build problems because state changes of the last invocation were lost.",
+                    file=sys.stderr)
+        except OSError as e:
+            print(colorize("Warning: cannot commit workspace state: "+str(e), WARNING),
+                file=sys.stderr)
+
+        try:
+            os.unlink(self.__uncommittedPath)
+        except FileNotFoundError:
+            pass
+        except OSError as e:
+            print(colorize("Warning: cannot delete corrupted state: "+str(e), WARNING),
+                file=sys.stderr)
+
     def __openBIdCache(self):
         if self.__buildIdCache is None:
             try:
@@ -160,6 +217,7 @@ class _BobState():
 
     def finalize(self):
         assert (self.__asynchronous == 0) and not self.__dirty
+        self.__commit(False)
         if self.__buildIdCache is not None:
             try:
                 self.__buildIdCache.execute("END")
@@ -167,21 +225,18 @@ class _BobState():
                 self.__buildIdCache.connection.close()
                 self.__buildIdCache = None
             except sqlite3.Error as e:
-                print(colorize("Warning: cannot commit buildid cache: "+str(e), "33"),
-                    file=stderr)
+                print(colorize("Warning: cannot commit buildid cache: "+str(e), WARNING),
+                    file=sys.stderr)
         if self.__lock:
             try:
                 os.unlink(self.__lock)
             except FileNotFoundError:
-                from .tty import colorize
-                from sys import stderr
-                print(colorize("Warning: lock file was deleted while Bob was still running!", "33"),
-                    file=stderr)
+                print(colorize("Warning: lock file was deleted while Bob was still running!",
+                               WARNING),
+                    file=sys.stderr)
             except OSError as e:
-                from .tty import colorize
-                from sys import stderr
-                print(colorize("Warning: cannot unlock workspace: "+str(e), "33"),
-                    file=stderr)
+                print(colorize("Warning: cannot unlock workspace: "+str(e), WARNING),
+                    file=sys.stderr)
 
     def setAsynchronous(self):
         self.__asynchronous += 1
@@ -265,6 +320,24 @@ class _BobState():
             self.__variantIds[path] = variantId
             self.__save()
 
+    def setStoragePath(self, workspace, storage):
+        """Set storage path for workspace.
+
+        Usually the workspace is also where the files are stored. Only if
+        workspace and storage path differ we store the actual storage location.
+        """
+        if storage == workspace: storage = None
+        if self.__storagePath.get(workspace) != storage:
+            if storage is None:
+                del self.__storagePath[workspace]
+            else:
+                self.__storagePath[workspace] = storage
+            self.__save()
+
+    def getStoragePath(self, workspace):
+        """Return diverted storage path for workspace (if any)."""
+        return self.__storagePath.get(workspace, workspace)
+
     def resetWorkspaceState(self, path, dirState):
         needSave = False
         if path in self.__results:
@@ -282,6 +355,9 @@ class _BobState():
         if path in self.__variantIds:
             del self.__variantIds[path]
             needSave = True
+        if path in self.__storagePath:
+            del self.__storagePath[path]
+            needSave = True
         if needSave:
             self.__save()
 
